import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Controller } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { UpdateUsernameCommand } from '../../commands/update-username.command';
import { Type } from '../../domain/dtos/type.enum';
import { UpdateUsernameRequest } from '../../domain/dtos/update-username.request';
import { DeleteAuthRequest } from '../../domain/dtos/delete-auth.request';
import { DeleteAuthCommand } from '../../commands/delete-auth.command';

@Controller()
export class AuthMessagerController {
  constructor(private readonly _commandBus: CommandBus) {}

  @RabbitSubscribe({
    exchange: 'user',
    routingKey: 'user.update',
  })
  public async userUpdatedHandler(message: string) {
    const updatedUser = JSON.parse(message);
    if (!updatedUser.hasOwnProperty('uuid')) throw new Error();
    if (updatedUser.hasOwnProperty('email')) {
      const updateUsernameRequest = new UpdateUsernameRequest();
      updateUsernameRequest.uuid = updatedUser.uuid;
      updateUsernameRequest.username = updatedUser.email;
      updateUsernameRequest.type = Type.EMAIL;
      this._commandBus.execute(
        new UpdateUsernameCommand(updateUsernameRequest),
      );
    }
    if (updatedUser.hasOwnProperty('phone')) {
      const updateUsernameRequest = new UpdateUsernameRequest();
      updateUsernameRequest.uuid = updatedUser.uuid;
      updateUsernameRequest.username = updatedUser.phone;
      updateUsernameRequest.type = Type.PHONE;
      this._commandBus.execute(
        new UpdateUsernameCommand(updateUsernameRequest),
      );
    }
  }

  @RabbitSubscribe({
    exchange: 'user',
    routingKey: 'user.delete',
  })
  public async userDeletedHandler(message: string) {
    const deletedUser = JSON.parse(message);
    if (!deletedUser.hasOwnProperty('uuid')) throw new Error();
    const deleteAuthRequest = new DeleteAuthRequest();
    deleteAuthRequest.uuid = deletedUser.uuid;
    this._commandBus.execute(new DeleteAuthCommand(deleteAuthRequest));
  }
}
